<?php
/*  Copyright (c) 2014 - 2020 ArcLindon LLC, All Rights Reserved
 *  $Id: citem_lib_citation.php 419 2020-06-28 17:44:33Z bill $
 *
 *  Changes:  
 *  2019/11/11  bherndon    v12.0.0.8d
 *  2019/11/02  bherndon    Added function citem_getFormatSpec() to support 
 *                          cached format spec arrays...conserving memory
 *  2019/11/01  bherndon    Same for citations in one particular circumstance
 *  2019/10/28  bherndon    Added source ID disambiguation by tree--passing the
 *                          DB 'gedcom' field as a parameter to most functions
 *  2019/02/28  bherndon    Added include for citem_options.php
 *  2018/09/08  bherndon    v11.1.2.8b
 *  2018/09/07  bherndon    Corrected default usage of periods in favor of 
 *                          proper trailers from the format specifications
 *  2018/04/07  bherndon    v11.1.2.8
 *  2018/04/07  bherndon    Added logic to support multiple separators 
 *  2018/04/07  bherndon    citem_doSeparator() now returns a boolean indicating
 *                          that trailing spaces should be surpressed on <br/>
 *  2018/02/25  bherndon    Now supporting a formatted reference via function
 *                          citem_getFormattedReference()
 *  2018/02/24  bherndon    Added a new seperator that creates a page break
 *                          for HTML (not supported for print)
 *  2018/01/22  bherndon    Made the 'mods' directory reference relative to 
 *                          config.php setting
 *  2018/01/21  bherndon    Added functions citem_doPrepend() and 
 *                          citem_doAppend() and web/PDF processing logic
 *  2017/02/09  bherndon    v11.1.0.7
 *  2017/02/08  bherndon    Now supporting variable extensions path
 *  2017/02/06  bherndon    Added logic to citem_formatSourceForWeb() to handle
 *                          dates an format them according to specifications
 *  2017/02/04  bherndon    Added custom GEDCOM tags to handle three additional
 *                          citation fields: description, citedat, and citetext.
 *  2016/02/13  bherndon    v10.1.0.6 (moved forward)
 *  2016/02/10  bherndon    Added tag TTLS for short titles and logic for 
 *                          $citem_use_short
 *  2016/01/23  bherndon    Rewrite of library code to reduce Mod Manager 
 *                          clashes on getCitations() function code
 *  2015/07/15  bherndon    v10.1.0.4
 *  2015/07/15  bherndon    Corrected global var naming issue that caused the
 *                          '_default' spec to be used continuously
 *  2015/07/05  bherndon    TNG 10.1.1 compatible
 *  2015/06/26  bherndon    Prefix modification "egenea_" => "citem_"
 *  2015/06/20  bherndon    Moving config vars to settings.php for greater 
 *                          flexibility
 *  2015/06/05  bherndon    Updates to support administrator interface
 *  2015/02/08  bherndon    v10.1.0.3 - TNG 10.1.0 compatible
 *  2015/02/08  bherndon    Updated queries to use TNG's new query wrapping 
 *                          library
 *  2014/12/06  bherndon    v10.0.2.3
 *  2014/12/05  bherndon    Now loading config file from the library to support
 *                          changing the file path via a mod parameter
 *  2014/11/17  bherndon    v10.0.2.2
 *  2014/11/16  bherndon    Added extra functions to handle print (PDF) 
 *                          formatting and removed simple format functions
 *  2014/11/10  bherndon    Added egenea_addSourceAsTags() and 
 *                          egenea_getCitationAsTags() to handle unsupported
 *                          citation fields (page, description, notes)
 *  2014/08/16  bherndon    Now supporting multiple typefaces per instruction
 *                          (an array of "typeface" strings)
 *  2014/08/07  bherndon    Updated format instruction names, added separator
 *                          'dash'
 *  2014/08/04  bherndon    TNG 9.2.2 compatible
 *  2014/07/22  bherndon    Created
 */
 
// Include the settings and parameters
include($cms['tngpath'] . "$extspath/citem_settings.php");
include($cms['tngpath'] . "$extspath/citem_options.php");

// Include the configuration file (update via config.php using Mod Manager)
// include($cms['tngpath'] . $citem_config_path);

// Other stuff needed by Cite Master
$citem_cfg_dir_rel = "$modspath/cite_master/config";
$citem_cfg_dir_abs = realpath($citem_cfg_dir_rel);

/*  Checks for a cached version of the format specification, and loads the 
 *  most recent if not found.  The cached version or recently loaded version
 *  is returned.
 *
 *  Returns:  The associative array representing the format specification
 */
function citem_getFormatSpec() {
  global $_SESSION, $citem_config_path, $citem_const_prev, 
         $citem_const_spec;
  
  // Check for an already loaded format specification
  if ($_SESSION[$citem_const_spec]) { 
    // we have a valid format specification
    if ($citem_config_path == $_SESSION[$citem_const_prev]) {
      // the current and previous paths match - assign cached format spec
      $citem_format_spec = $_SESSION[$citem_const_spec];
    } else {
      // paths didn't match - mod parameter was changed
      include($cms['tngpath'] . $citem_config_path);  
      $_SESSION[$citem_const_prev] = $citem_config_path;
      $_SESSION[$citem_const_spec] = $citem_format_spec;
    }      
  } else {
    // no cached specification 
    include($cms['tngpath'] . $citem_config_path);  
    $_SESSION[$citem_const_prev] = $citem_config_path;
    $_SESSION[$citem_const_spec] = $citem_format_spec;
  }
  
  return $citem_format_spec;
}

/*  Validate the path of a configration file as "ok" for writing
 *  $file:    A file path, typically relative to the TNG root but could
 *            be absolute.
 *  Returns:  <true> when valid, <false> otherwise
 */
function citem_validPath($path) {
  return ($citem_cfg_dir_abs == realpath(dirname($path)));
}

/*  Given a known key value, returns the text with the selected surrounding
 *  characters.  Valid selectors are: 'quotes', 'parens', 'brackets'
 *            (i.e., square brackets), 'braces' (i.e., curly braces), and 
 *            'angles' (i.e., angle brackets).
 *  
 *  $key:     A selector
 *  $text:    The text that will be surrounded by the chosen characters
 *
 *  Returns:  A string
 *  Contract: The default behavior is to return the text unmodified.
 */
function citem_doSurround($key, $text) {
  switch ($key) {
    case "quotes":
    case "doubles":  $retval = "&quot;$text&quot;"; break;
    case "singles":  $retval = "'$text'"; break;
    case "parens":   $retval = "($text)"; break;
    case "brackets": $retval = "&#91;$text&#93;"; break;
    case "braces":   $retval = "&#123;$text&#125;"; break;
    case "angles":   $retval = "&lt;$text&gt;"; break;
    default:         $retval = $text;
  }
  
  return $retval;
}

/*  Given a known key value, returns the text with the selected typeface 
 *  applied as HTML.  Valid selectors are: 'italic', 'bold', 'underline', 
 *            and 'plain'.
 *  
 *  $key:     A selector
 *  $text:    The text to which the selected typeface will be applied
 *
 *  Returns:  A string
 *  Contract: 'plain' is the default and is used if the key is not known.
 */
function citem_doTypeface($key, $text) {
  switch ($key) {
    case "italic":    $retval = "<i>$text</i>"; break;
    case "bold":      $retval = "<b>$text</b>"; break;
    case "uline":     $retval = "<u>$text</u>"; break; 
    case "plain":     $retval = $text; break;
    default:          $retval = $text;
  }

  return $retval;
}  

/*  Returns the input text with the 'Prepend' value prepended 
 *  
 *  $prepend:  A value to prepend
 *  $text:    The text to which the 'prepend; will be prepended
 *
 *  Returns:  A string
 */
function citem_doPrepend($prepend, $text) {
  return $prepend . $text;
}  

/*  Returns the input text with the 'append' value appended 
 *  
 *  $append:  A value to append
 *  $text:    The text to which the 'append' will be appended
 *
 *  Returns:  A string
 */
function citem_doAppend($append, $text) {
  return $text . $append;
}  

/*  Given a known key value, returns the text with the selected separator 
 *  appended.  The valid key selectors are: 'comma', 'space', 'period', 
 *            'colon', 'dash' and 'semicolon'.
 *  
 *  $key:     A selector
 *  $text:    The text to which the selected separator will be appended
 *  $supress: A referenced boolean. Set to true, indicates to the caller
 *            that spaces should be supressed after a <br/>
 *  $pad:     (future) Whether or not the seperator character should be padded 
 *            with spaces
 *
 *  Returns:  A string
 *  Contract: The default behavior is to return the text unmodified.
 */
function citem_doSeparator($key, $text, &$supress, $pad = 0) {
  switch ($key) {
    case "comma":     $retval = $text . ","; break;
    case "space":     $retval = $text . "&nbsp;"; 
                      $supress = true;
                      break;
    case "period":    $retval = $text . "."; break;
    case "colon":     $retval = $text . ":"; break;
    case "dash":      $retval = $text . "-"; break;
    case "semi":      $retval = $text . ";"; break;
    case "break":     $retval = $text . "<br/>"; 
                      $supress = true;
                      break;
    default:          $retval = $text;
  }
  
  return $retval;
} 

/*  Given a known key value, returns the text with the selected surrounding
 *  characters.  Valid selectors are: 'quotes', 'parens', 'brackets'
 *            (i.e., square brackets), 'braces' (i.e., curly braces), and 
 *            'angles' (i.e., angle brackets).
 *  
 *  $key:     A selector
 *  $text:    The text that will be surrounded by the chosen characters
 *
 *  Returns:  A string
 *  Contract: The default behavior is to return the text unmodified.  Note also
 *            that there is no way to support angle brackets in PDF.
 */
function citem_doPrintSurround($key, $text) {
  switch ($key) {
    case "quotes":   $retval = "\"$text\""; break;
    case "parens":   $retval = "($text)"; break;
    case "brackets": $retval = "[$text]"; break;
    case "braces":   $retval = "{" . $text. "}"; break;
    case "angles":   $retval = $text; break;
    default:         $retval = $text;
  }
  
  return $retval;
}

/*  Given a known key value, returns the text with the selected typeface 
 *  applied as HTML.  Valid selectors are: 'italic', 'bold', 'underline', 
 *            and 'plain'.
 *  
 *  $key:     A selector
 *  $text:    The text to which the selected typeface will be applied
 *
 *  Returns:  A string
 *  Contract: 'plain' is the default and is used if the key is not known.
 */
function citem_doPrintTypeface($key, $text) {
  switch ($key) {
    case "uline":     $retval = "_" . $text . "_"; break; 
    default:          $retval = $text;
  }

  return $retval;
}  

/*  Given a known key value, returns the text with the selected separator 
 *  appended.  The valid key selectors are: 'comma', 'space', 'period', 
 *            'colon', 'dash' and 'semicolon'.
 *  
 *  $key:     A selector
 *  $text:    The text to which the selected separator will be appended
 *  $pad:     (future) Whether or not the seperator character should be padded 
 *            with spaces
 *
 *  Returns:  A string
 *  Contract: The default behavior is to return the text unmodified.
 */
function citem_doPrintSeparator($key, $text, $pad = 0) {
  switch ($key) {
    case "comma":     $retval = $text . ","; break;
    case "space":     $retval = $text . " "; break;
    case "period":    $retval = $text . "."; break;
    case "colon":     $retval = $text . ":"; break;
    case "dash":      $retval = $text . "-"; break;
    case "semi":      $retval = $text . ";"; break;
    default:          $retval = $text;
  }
  
  return $retval;
}    

/*  Given a valid source ID, adds all the known information about the source
 *  (as GEDCOM tags) to the input associative array.
 *  
 *  $srcID:   A source identifier 
 *  $srcTree: The tree associated with the source identifier
 *  $result:  An array to add results to
 *
 *  Returns:  (none)
 *
 *  Contract: If there are any duplicates source fields that already exist
 *            in the input array, they will be overridden buy new values from
 *            the sources table.  Effectively, this means that preference is
 *            given to fields from the sources table.
 */
function citem_addSourceAsTags($srcID, $srcTree, &$result) {
	global $events_table, $sources_table, $repositories_table, 
	       $eventtypes_table, $text, $tree, $citem_use_short;
 
	$rtncnt = 0;
		
	// prep the query for the standard fields
	if ($srcTree)
    $query = "SELECT sourceID, title, shorttitle, author, publisher, actualtext,
                     reponame, $sources_table.repoID as repoID, callnum, other
              FROM       $sources_table LEFT JOIN $repositories_table
                         on $sources_table.repoID = $repositories_table.repoID 
                     AND $sources_table.gedcom = $repositories_table.gedcom 
              WHERE      $sources_table.sourceID = \"$srcID\"
                     AND $sources_table.gedcom = \"$srcTree\"";  
	else
    $query = "SELECT sourceID, title, shorttitle, author, publisher, actualtext,
                     reponame, $sources_table.repoID as repoID, callnum, other
              FROM       $sources_table LEFT JOIN $repositories_table
                         on $sources_table.repoID = $repositories_table.repoID 
                     AND $sources_table.gedcom = $repositories_table.gedcom 
              WHERE      $sources_table.sourceID = \"$srcID\"";
	
  // execute the query for the standard events
  $stdevents = tng_query($query);
               
  // there should only be one association, so we just fetch the first
  $stdevent = tng_fetch_assoc($stdevents);
  
  // handle the 8 supported standard fields - convert to GEDCOM associations
  if ($stdevent['title']) 
    $result['TITL'] = array("info" => $stdevent['title']);
  if ($stdevent['shorttitle'])
    $result['TTLS'] = array("info" => $stdevent['shorttitle']);
  if ($stdevent['publisher']) 
    $result['PUBL'] = array("info" => $stdevent['publisher']); 
  if ($stdevent['author']) 
    $result['AUTH'] = array("info" => $stdevent['author']);
  if ($stdevent['reponame']) 
    $result['REPO'] = array("info" => $stdevent['reponame']);
  if ($stdevent['actualtext']) 
    $result['TEXT'] = array("info" => $stdevent['actualtext']);
  if ($stdevent['callnum']) 
    $result['CALN'] = array("info" => $stdevent['callnum']);
  if ($stdevent['quay']) 
    $result['QUAY'] = array("info" => $stdevent['quay']);

  // now patch up the titles according to the flag
  if (($stdevent['title'] == null) and ($citem_use_short == 'yes'))
    $result['TITL'] = array("info" => $stdevent['shorttitle']);
  
  // release the cursor for the standard fields
  tng_free_result($stdevents);
 
  // prep the query for the 'custom' event fields
  if ($srcTree)
    $query = "SELECT display, eventdate, eventdatetr, eventplace, age, agency, 
                     cause, addressID, info, tag, description, eventID 
              FROM   ($events_table, $eventtypes_table) 
              WHERE      persfamID = \"$srcID\" 
                     AND $events_table.eventtypeID = $eventtypes_table.eventtypeID 
                     AND gedcom = \"$srcTree\" 
                     AND keep = \"1\" 
                     AND parenttag = \"\" 
              ORDER BY ordernum, tag, description, eventdatetr, info, eventID";
	else        
    $query = "SELECT display, eventdate, eventdatetr, eventplace, age, agency, 
                     cause, addressID, info, tag, description, eventID 
              FROM   ($events_table, $eventtypes_table) 
              WHERE      persfamID = \"$srcID\" 
                     AND $events_table.eventtypeID = $eventtypes_table.eventtypeID 
                     AND keep = \"1\" 
                     AND parenttag = \"\" 
              ORDER BY ordernum, tag, description, eventdatetr, info, eventID";
  
  // execute the query for the custom events
	$custevents = tng_query($query);
	              
	while ($custevent = tng_fetch_assoc($custevents))	{
	  $result[$custevent['tag']] = $custevent;
	}

  // release the cursor for the custom fields
  tng_free_result($custevents);
}

/*  Given a valid source ID, fetches all the known information about the source
 *  (as GEDCOM tags) and returns it as an associative array.
 *  
 *  $srcID:   A source identifier 
 *  $srcTree: The tree associated with the source identifier (for 
 *            disambinguation)
 *
 *  Returns:  An associative array
 */
function citem_getSourceAsTags($srcID, $srcTree) {
	global $events_table, $sources_table, $repositories_table, 
	       $eventtypes_table, $text, $tree;
 
	$result = array();
	$rtncnt = 0;
	
  // now add the source and repository information when requested
  citem_addSourceAsTags($srcID, $srcTree, $result);
  
  return $result;
}

/*  Given a valid citation ID, returns all the known information about the
 *  citation as an associative array that uses GEDCOM tags as keys.
 *  
 *  $citeID:  A citation identifier 
 *  $addSrc:  A flag indicating if source data should be added to the result 
 *            (if <non-null>) or should not be added (if <null>, the default).
 *
 *  Returns:  An associative array
 */
function citem_getCitationAsTags($citeID, $addSrc = null) {
	global $events_table, $sources_table, $repositories_table, 
	       $citations_table, $eventtypes_table, $text, $tree;
 
	$result = array();
	$rtncnt = 0;
	
	// prep the query for the standard fields
	$query = "SELECT  citationID, sourceID, gedcom, page, note, quay, description, 
	                  citetext, citedate, citedatetr
	          FROM    $citations_table
	          WHERE   citationID = \"$citeID\"";
	          
	// execute the query for the citation standard fields
  $citeevents = tng_query($query);
               
  // there should only be one association, so we just fetch the first
  $citeevent = tng_fetch_assoc($citeevents);

  // handle the 3 supported citation fields - convert to GEDCOM associations
  if ($citeevent['page']) 
    $result['PAGE'] = array("info" => $citeevent['page']); 
  if ($citeevent['note']) 
    $result['NOTE'] = array("info" => $citeevent['note']); 
  if ($citeevent['quay']) 
    $result['QUAY'] = array("info" => $citeevent['quay']); 
  if ($citeevent['description']) 
    $result['_CDSC'] = array("info" => $citeevent['description']); 
  if ($citeevent['citetext']) 
    $result['_CTXT'] = array("info" => $citeevent['citetext']); 
  if ($citeevent['citedate']) 
    $result['_CDTE'] = array("info" => $citeevent['citedatetr']); 


  if ($addSrc) {
    // now add the source and repository information when requested
    citem_addSourceAsTags($citeevent['sourceID'], $citeevent['gedcom'], 
                          $result);
  }
 
  // release the cursor for the citation fields
  tng_free_result($citeevents);
  
  // return all the extracted citation fields as GEDCOM associations
	return $result;
}

/*  Generates and returns an array of information needed for generating page
 *  references for individual and family pages.
 *  
 *  $label:   A preformatted label for the page reference--typically an 
 *            individual's full name or a family label.
 *
 *  Returns:  An associative array
 */
function citem_getRefdataAsTags($label) {
	global $_SYSTEM, $citem_author, $sitename;
 
	$result = array();
	
	// generate the full URL for the current page
	$url    = (isset($_SERVER['HTTPS']) ? "https" : "http") 
	          . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
	
	$result['_RNAM'] = array("info" => $label);
  $result['_RAUT'] = array("info" => $citem_author);
  $result['_RTTL'] = array("info" => $sitename);
  $result['_RURL'] = array("info" => $url);
  $result['_RCDT'] = array("info" => date(DATE_RFC822));
  
  // return all the extracted citation fields as GEDCOM associations
	return $result;
}

/*  Prunes and returns a format array according to the data tags found in a 
 *  data array.  The function removes any format tags not found in the data
 *  array.
 *  
 *  $format:  The associative array of format tags (GEDCOM) and format 
 *            instructions.  (See file citem_config_test.php for examples.)
 *  $srcData: The associative array of source data typically supplied by a call
 *            to citem_getSourceAsTags().
 *
 *  Returns:  A 'pruned' format array
 */
function citem_getPruneFmt($format, $data)
{
  $result = array();
  
  foreach ($format as $tag => $elements) {
    if ($data[$tag]['info'])
      // we have actual data for this format tag - include in pruned result
      $result[$tag] = $elements;
  }
  
  return $result;
}

/*  Given valid arrays of formatting data and source data, formats and returns
 *  the source.  Optional 'leader' and 'trailer' variables can be used to add 
 *  prepended and appended data to the output.  Context indicates the formatting
 *  method that should be used.
 *  
 *  $srcFmt:  A associative array of format tags (GEDCOM) and format 
 *            instructions.  (See file citem_config_test.php for examples.)
 *  $srcData: A associative array of source data typicall supplied by a call
 *            to citem_getSourceAsTags().
 *  $leader:  (optional) Data to be prepended to the output
 *  $trailer: (optional) Data to be appended to the output 
 *  $context: One of "web" (HTML compatible presentation) or "pdf" (simplified,
 *            printable presentation)
 *
 *  Returns:  A string representing the formatted source
 */
function citem_formatSource($srcFmt, $srcData, $leader = "", $trailer = "",
                             $context = "web") 
{
  $result = "";
  
  switch ($context) {
    case "pdf":   $result = citem_formatSourceForPDF($srcFmt, $srcData, 
                              $leader, $trailer); break;
    default:      $result = citem_formatSourceForWeb($srcFmt, $srcData,
                              $leader, $trailer);
  }
  
  // return the source, formatted for the context
  return $result;
}
  
/*  Given valid arrays of formatting data and source data, formats the source
 *  for the web using valid HTML.  Optional 'leader' and 'trailer' variables 
 *  can be used to add prepended and appended data to the output.
 *  
 *  $srcFmt:  A associative array of format tags (GEDCOM) and format 
 *            instructions.  (See file citem_config_test.php for examples.)
 *  $srcData: A associative array of source data typicall supplied by a call
 *            to citem_getSourceAsTags().
 *  $leader:  (optional) Data to be prepended to the output
 *  $trailer: (optional) Data to be appended to the output 
 *
 *  Returns:  A string representing the formatted source
 */
function citem_formatSourceForWeb($srcFmt, $srcData, $leader = "", 
                                   $trailer = "") 
{
  $result    = "";
  $pruneFmt  = citem_getPruneFmt($srcFmt, $srcData);
  $srcFmtCnt = count($pruneFmt);
  
  // iterate across the GEDCOM tags identified in the pre-pruned format 
  foreach($pruneFmt as $tag => $elements) {
    if ($srcData[$tag]) {
      // there's source data for this format tag - get the contents
      $content = $srcData[$tag]['info'];
          
      if ($content) {
        // set the space surpression flag
        $supress = false;
        
        // we have content for this tag - format it
        $mod = $content;
        
        // iterate across the formatting elements
        foreach ($elements as $category => $variant) {
          if ($category == "format") {
            // a date formatting instruction
            $date = date_create($content);
            $mod  = date_format($date, $variant);
          }
          if ($category == "typeface") {
          	if (is_array($variant)) {
          		// an array of typefaces
          		foreach ($variant as $typeface) {
          			$mod = citem_doTypeface($typeface, $mod);
          		}
          	} else {
          		// simple single typeface
              $mod = citem_doTypeface($variant, $mod);
          	}
          } 
          if ($category == "prepend")
            $mod = citem_doPrepend($variant, $mod);
          if ($category == "append")
            $mod = citem_doAppend($variant, $mod);
          if (($category == "separator") and ($srcFmtCnt > 1)) {
            // we supress if this is the last format element we're working on
          	if (is_array($variant)) {
          		// an array of separators
          		foreach ($variant as $separator) {
                $mod = citem_doSeparator($separator, $mod, $supress);
          		}
          	} else {
          		// simple single separator
              $mod = citem_doSeparator($variant, $mod, $supress);
          	}
          }
          if ($category == "surround") 
            $mod = citem_doSurround($variant, $mod);
        }
      }
      
      $result .= $mod;
      $result .=  (($srcFmtCnt > 1) and (! $supress)) ? "&nbsp;" : "";
      
      // decriment the element count
      $srcFmtCnt--;
    }
  }
  
  $result = citem_doSeparator($trailer, $result, $supress);   

  // return the formatted source content
  return $result;
}

/*  Given valid arrays of formatting data and source data, formats the source
 *  for display as PDF.  Optional 'leader' and 'trailer' variables 
 *  can be used to add prepended and appended data to the output.
 *  
 *  $srcFmt:  A associative array of format tags (GEDCOM) and format 
 *            instructions.  (See file citem_config_test.php for examples.)
 *  $srcData: A associative array of source data typicall supplied by a call
 *            to citem_getSourceAsTags().
 *  $leader:  (optional) Data to be prepended to the output
 *  $trailer: (optional) Data to be appended to the output 
 *
 *  Returns:  A string representing the formatted source
 */
function citem_formatSourceForPDF($srcFmt, $srcData, $leader = "", 
                                   $trailer = "") 
{
  $result = "";
  $pruneFmt = citem_getPruneFmt($srcFmt, $srcData);
  $srcFmtCnt = count($pruneFmt);
  
  // iterate across the GEDCOM tags identified in the pre-pruned format 
  foreach($pruneFmt as $tag => $elements) {
    if ($srcData[$tag]) {
      // there's source data for this format tag - get the contents
      $content = $srcData[$tag]['info'];
          
      if ($content) {
        // we have content for this tag - format it
        $mod = $content;
        
        // iterate across the formatting elements
        foreach ($elements as $category => $variant) {
          if ($category == "format") {
            // a date formatting instruction
            $date = date_create($content);
            $mod  = date_format($date, $variant);
          }
          if ($category == "typeface") {
          	if (is_array($variant)) {
          		// an array of typefaces
          		foreach ($variant as $typeface) {
          			$mod = citem_doPrintTypeface($typeface, $mod);
          		}
          	} else {
          		// simple single typeface
              $mod = citem_doPrintTypeface($variant, $mod);
          	}
          } 
          if ($category == "prepend")
            $mod = citem_doPrepend($variant, $mod);
          if ($category == "append")
            $mod = citem_doAppend($variant, $mod);
          if (($category == "separator") and ($srcFmtCnt > 1))
            // we supress if this is the last format element we're working on
            $mod = citem_doPrintSeparator($variant, $mod);
          if ($category == "surround") 
            $mod = citem_doPrintSurround($variant, $mod);
        }
      }
      
      $result .= $mod;
      $result .=  ($srcFmtCnt > 1) ? " " : "";
      
      // decriment the element count
      $srcFmtCnt--;
    }
  }
  
  $result = citem_doPrintSeparator($trailer, $result);   

  // return the formatted source content
  return $result;
}

/*  Given a known source ID, returns the source's data formatted according
 *  the pre-prepared format defined in current Citation Master configuration
 *  file for the source type in question.
 *  
 *  $srcID:   The source identifier (A string of the format 'SXYZ' as in 'S012')
 *  $srcTree: The tree associated with the source identifier (for 
 *            disambiguation)
 *  $context: A qualifier that defines the context in which the data will be 
 *            printed.  (Used to choose between formating methods.) 
 *
 *  Returns:  A string representing the formatted source
 */
function citem_getFormattedSource($srcID, $srcTree, $context = "web") {
  global $citem_type_tag;
  
  $fmtSpec = citem_getFormatSpec();
  $srcData = citem_getSourceAsTags($srcID, $srcTree);
  $srcType = $srcData[$citem_type_tag]['info'];
  
  // get a valid source format or the 'default format and assign instruction
  if ($fmtSpec[$srcType]) {
    // assign format instructions tags and trailer from the selected type
    $srcFmt  = $fmtSpec[$srcType]['tags'];
    $trailer = $fmtSpec[$srcType]['trailer'];  
  } else {
    // assing format instruction tags and trailer from the '_default' set
    $srcFmt  = $fmtSpec['_default']['tags'];
    $trailer = $fmtSpec['_default']['trailer'];  
  }

  // get a formatted source and return it
  $pretty = citem_formatSource($srcFmt, $srcData, "", $trailer, $context);
  return $pretty;
}  

/*  Given a known citation ID, returns the citation and source data formatted 
 *  according the pre-prepared format defined in the current Citation Master
 *  configuration file for the source type in question.
 *  
 *  $citeID:  The citation identifier
 *  $context: A qualifier that defines the context in which the data will be 
 *            printed.  (Used to choose between formating methods.) 
 *
 *  Returns:  A string representing the formatted source
 */
function citem_getFormattedCitation($citeID, $context = "web") {
  global $citem_type_tag;
  
  $fmtSpec  = citem_getFormatSpec();
  $citeData = citem_getCitationAsTags($citeID, 1);
  $srcType  = $citeData[$citem_type_tag]['info'];
  
  // get a valid source format or the 'default format and assign instruction
  if ($fmtSpec[$srcType]) {
    // assign format instructions tags and trailer from the selected type
    $srcFmt  = $fmtSpec[$srcType]['tags'];
    $trailer = $fmtSpec[$srcType]['trailer'];  
  } else {
    // assing format instruction tags and trailer from the '_default' set
    $srcFmt  = $fmtSpec['_default']['tags'];
    $trailer = $fmtSpec['_default']['trailer'];  
  }

  // get a formatted source and return it
  $pretty = citem_formatSource($srcFmt, $citeData, "", $trailer, $context);
  return $pretty;
}  

/*  Given a known citation ID, returns the citation and source data formatted 
 *  according the pre-prepared format defined in the current Citation Master
 *  configuration file for the source type in question.
 *  
 *  $label:   Preformatted label for the page--typically the name of the 
 *            individual or family title referenced on the page.
 *  $context: A qualifier that defines the context in which the data will be 
 *            printed.  (Used to choose between formating methods.) 
 *
 *  Returns:  A string representing the formatted reference
 */
function citem_getFormattedReference($label, $context = "web") {
  global $citem_author, $citem_sourceid;
  
  // get the standard reference data...
  $refData = citem_getRefdataAsTags($label);
  $fmtSpec = citem_getFormatSpec();
  
  // add data from the source identified by mod parameters
  citem_addSourceAsTags($citem_sourceid, null, $refData);

  // get the 'reference' format tag set and trailer
  $srcFmt  = $fmtSpec['_reference']['tags'];
  $trailer = $fmtSpec['_reference']['trailer'];
  
  // get a formatted page reference and return it
  $pretty = $srcFmt
            ? citem_formatSource($srcFmt, $refData, "", $trailer, "web")
            : "[ None ]";
   
  return $pretty;
}  
?>